/*
 * Copyright 2015 Matt Parsons
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package br.com.kaiquecruz.wifiwizard;

import org.apache.cordova.*;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.net.NetworkInfo;
import android.net.wifi.WifiManager;
import android.net.wifi.WifiConfiguration;
import android.net.wifi.WifiEnterpriseConfig;
import android.net.wifi.ScanResult;
import android.net.wifi.WifiInfo;
import android.net.wifi.SupplicantState;
import android.content.Context;
import android.util.Log;


public class WifiWizard extends CordovaPlugin {
    
    private static final String ADD_NETWORK = "addNetwork";
    private static final String REMOVE_NETWORK = "removeNetwork";
    private static final String CONNECT_NETWORK = "connectNetwork";
    private static final String DISCONNECT_NETWORK = "disconnectNetwork";
    private static final String DISCONNECT = "disconnect";
    private static final String LIST_NETWORKS = "listNetworks";
    private static final String START_SCAN = "startScan";
    private static final String GET_SCAN_RESULTS = "getScanResults";
    private static final String GET_CONNECTED_SSID = "getConnectedSSID";
    private static final String GET_CONNECTED_BSSID = "getConnectedBSSID";
    private static final String IS_WIFI_ENABLED = "isWifiEnabled";
    private static final String SET_WIFI_ENABLED = "setWifiEnabled";
    private static final String SAVE_EAP_CONFIG = "saveEapConfig";
    private static final String TAG = "WifiWizard";
    
    private WifiManager wifiManager;
    private CallbackContext callbackContext;
    
    @Override
    public void initialize(CordovaInterface cordova, CordovaWebView webView) {
        super.initialize(cordova, webView);
        this.wifiManager = (WifiManager) cordova.getActivity().getSystemService(Context.WIFI_SERVICE);
    }
    
    @Override
    public boolean execute(String action, JSONArray data, CallbackContext callbackContext)
      throws JSONException {
        
        this.callbackContext = callbackContext;
        
        if(action.equals(IS_WIFI_ENABLED)) {
            return this.isWifiEnabled(callbackContext);
        }
        else if(action.equals(SET_WIFI_ENABLED)) {
            return this.setWifiEnabled(callbackContext, data);
        }
        else if (!wifiManager.isWifiEnabled()) {
            callbackContext.error("Wifi is not enabled.");
            return false;
        }
        else if(action.equals(ADD_NETWORK)) {
            return this.addNetwork(callbackContext, data);
        }
        else if(action.equals(REMOVE_NETWORK)) {
            return this.removeNetwork(callbackContext, data);
        }
        else if(action.equals(CONNECT_NETWORK)) {
            return this.connectNetwork(callbackContext, data);
        }
        else if(action.equals(DISCONNECT_NETWORK)) {
            return this.disconnectNetwork(callbackContext, data);
        }
        else if(action.equals(LIST_NETWORKS)) {
            return this.listNetworks(callbackContext);
        }
        else if(action.equals(START_SCAN)) {
            return this.startScan(callbackContext);
        }
        else if(action.equals(GET_SCAN_RESULTS)) {
            return this.getScanResults(callbackContext, data);
        }
        else if(action.equals(DISCONNECT)) {
            return this.disconnect(callbackContext);
        }
        else if(action.equals(GET_CONNECTED_SSID)) {
            return this.getConnectedSSID(callbackContext);
        }
        else if(action.equals(GET_CONNECTED_BSSID)) {
            return this.getConnectedBSSID(callbackContext);
        }
        else if(action.equals(SAVE_EAP_CONFIG)) {
            return this.saveEapConfig(callbackContext, data);
        }
        else {
            callbackContext.error("Incorrect action parameter: " + action);
        }
        
        return false;
    }
    
    /**
     * This methods adds a network to the list of available WiFi networks.
     * If the network already exists, then it updates it.
     *
     * @params callbackContext     A Cordova callback context.
     * @params data                JSON Array with [0] == SSID, [1] == password
     * @return true    if add successful, false if add fails
     */
    private boolean addNetwork(CallbackContext callbackContext, JSONArray data) {
        // Initialize the WifiConfiguration object
        WifiConfiguration wifi = new WifiConfiguration();
        
        Log.d(TAG, "WifiWizard: addNetwork entered.");
        
        try {
            // data's order for ANY object is 0: ssid, 1: authentication algorithm,
            // 2+: authentication information.
            String authType = data.getString(1);
            
            
            if (authType.equals("WPA")) {
                // WPA Data format:
                // 0: ssid
                // 1: auth
                // 2: password
                String newSSID = data.getString(0);
                wifi.SSID = newSSID;
                String newPass = data.getString(2);
                wifi.preSharedKey = newPass;
                
                wifi.status = WifiConfiguration.Status.ENABLED;
                wifi.allowedGroupCiphers.set(WifiConfiguration.GroupCipher.TKIP);
                wifi.allowedGroupCiphers.set(WifiConfiguration.GroupCipher.CCMP);
                wifi.allowedKeyManagement.set(WifiConfiguration.KeyMgmt.WPA_PSK);
                wifi.allowedPairwiseCiphers.set(WifiConfiguration.PairwiseCipher.TKIP);
                wifi.allowedPairwiseCiphers.set(WifiConfiguration.PairwiseCipher.CCMP);
                wifi.allowedProtocols.set(WifiConfiguration.Protocol.RSN);
                wifi.allowedProtocols.set(WifiConfiguration.Protocol.WPA);
                
                wifi.networkId = ssidToNetworkId(newSSID);
                
                if ( wifi.networkId == -1 ) {
                    wifiManager.addNetwork(wifi);
                    callbackContext.success(newSSID + " successfully added.");
                }
                else {
                    wifiManager.updateNetwork(wifi);
                    callbackContext.success(newSSID + " successfully updated.");
                }
                
                wifiManager.saveConfiguration();
                return true;
            }
            else if (authType.equals("WEP")) {
                // TODO: connect/configure for WEP
                Log.d(TAG, "WEP unsupported.");
                callbackContext.error("WEP unsupported");
                return false;
            }
            else if (authType.equals("NONE")) {
                String newSSID = data.getString(0);
                wifi.SSID = newSSID;
                wifi.allowedKeyManagement.set(WifiConfiguration.KeyMgmt.NONE);
                wifi.networkId = ssidToNetworkId(newSSID);
                
                if ( wifi.networkId == -1 ) {
                    wifiManager.addNetwork(wifi);
                    callbackContext.success(newSSID + " successfully added.");
                }
                else {
                    wifiManager.updateNetwork(wifi);
                    callbackContext.success(newSSID + " successfully updated.");
                }
                
                wifiManager.saveConfiguration();
                return true;
            }
            // TODO: Add more authentications as necessary
            else {
                Log.d(TAG, "Wifi Authentication Type Not Supported.");
                callbackContext.error("Wifi Authentication Type Not Supported: " + authType);
                return false;
            }
        }
        catch (Exception e) {
            callbackContext.error(e.getMessage());
            Log.d(TAG,e.getMessage());
            return false;
        }
    }
    
    /**
     *    This method removes a network from the list of configured networks.
     *
     *    @param    callbackContext        A Cordova callback context
     *    @param    data                JSON Array, with [0] being SSID to remove
     *    @return    true if network removed, false if failed
     */
    private boolean removeNetwork(CallbackContext callbackContext, JSONArray data) {
        Log.d(TAG, "WifiWizard: removeNetwork entered.");
        
        if(!validateData(data)) {
            callbackContext.error("WifiWizard: removeNetwork data invalid");
            Log.d(TAG, "WifiWizard: removeNetwork data invalid");
            return false;
        }
        
        // TODO: Verify the type of data!
        try {
            String ssidToDisconnect = data.getString(0);
            
            int networkIdToRemove = ssidToNetworkId(ssidToDisconnect);
            
            if (networkIdToRemove >= 0) {
                wifiManager.removeNetwork(networkIdToRemove);
                wifiManager.saveConfiguration();
                callbackContext.success("Network removed.");
                return true;
            }
            else {
                callbackContext.error("Network not found.");
                Log.d(TAG, "WifiWizard: Network not found, can't remove.");
                return false;
            }
        }
        catch (Exception e) {
            callbackContext.error(e.getMessage());
            Log.d(TAG, e.getMessage());
            return false;
        }
    }
    
    /**
     *    This method connects a network.
     *
     *    @param    callbackContext        A Cordova callback context
     *    @param    data                JSON Array, with [0] being SSID to connect
     *    @return    true if network connected, false if failed
     */
    private boolean connectNetwork(CallbackContext callbackContext, JSONArray data) {
        Log.d(TAG, "WifiWizard: connectNetwork entered.");
        if(!validateData(data)) {
            callbackContext.error("WifiWizard: connectNetwork invalid data");
            Log.d(TAG, "WifiWizard: connectNetwork invalid data.");
            return false;
        }
        String ssidToConnect = "";
        
        try {
            ssidToConnect = data.getString(0);
        }
        catch (Exception e) {
            callbackContext.error(e.getMessage());
            Log.d(TAG, e.getMessage());
            return false;
        }
        
        int networkIdToConnect = ssidToNetworkId(ssidToConnect);
        
        if (networkIdToConnect >= 0) {
            // We disable the network before connecting, because if this was the last connection before
            // a disconnect(), this will not reconnect.
            wifiManager.disableNetwork(networkIdToConnect);
            wifiManager.enableNetwork(networkIdToConnect, true);
            
            SupplicantState supState;
            WifiInfo wifiInfo = wifiManager.getConnectionInfo();
            supState = wifiInfo.getSupplicantState();
            callbackContext.success(supState.toString());
            return true;
            
        }else{
            callbackContext.error("WifiWizard: cannot connect to network");
            return false;
        }
    }
    
    /**
     *    This method disconnects a network.
     *
     *    @param    callbackContext        A Cordova callback context
     *    @param    data                JSON Array, with [0] being SSID to connect
     *    @return    true if network disconnected, false if failed
     */
    private boolean disconnectNetwork(CallbackContext callbackContext, JSONArray data) {
        Log.d(TAG, "WifiWizard: disconnectNetwork entered.");
        if(!validateData(data)) {
            callbackContext.error("WifiWizard: disconnectNetwork invalid data");
            Log.d(TAG, "WifiWizard: disconnectNetwork invalid data");
            return false;
        }
        String ssidToDisconnect = "";
        // TODO: Verify type of data here!
        try {
            ssidToDisconnect = data.getString(0);
        }
        catch (Exception e) {
            callbackContext.error(e.getMessage());
            Log.d(TAG, e.getMessage());
            return false;
        }
        
        int networkIdToDisconnect = ssidToNetworkId(ssidToDisconnect);
        
        if (networkIdToDisconnect > 0) {
            wifiManager.disableNetwork(networkIdToDisconnect);
            callbackContext.success("Network " + ssidToDisconnect + " disconnected!");
            return true;
        }
        else {
            callbackContext.error("Network " + ssidToDisconnect + " not found!");
            Log.d(TAG, "WifiWizard: Network not found to disconnect.");
            return false;
        }
    }
    
    /**
     *    This method disconnects current network.
     *
     *    @param    callbackContext        A Cordova callback context
     *    @return    true if network disconnected, false if failed
     */
    private boolean disconnect(CallbackContext callbackContext) {
        Log.d(TAG, "WifiWizard: disconnect entered.");
        if (wifiManager.disconnect()) {
            callbackContext.success("Disconnected from current network");
            return true;
        } else {
            callbackContext.error("Unable to disconnect from the current network");
            return false;
        }
    }
    
    /**
     *    This method uses the callbackContext.success method to send a JSONArray
     *    of the currently configured networks.
     *
     *    @param    callbackContext        A Cordova callback context
     *    @param    data                JSON Array, with [0] being SSID to connect
     *    @return    true if network disconnected, false if failed
     */
    private boolean listNetworks(CallbackContext callbackContext) {
        Log.d(TAG, "WifiWizard: listNetworks entered.");
        List<WifiConfiguration> wifiList = wifiManager.getConfiguredNetworks();
        
        JSONArray returnList = new JSONArray();
        
        for (WifiConfiguration wifi : wifiList) {
            returnList.put(wifi.SSID);
        }
        
        callbackContext.success(returnList);
        
        return true;
    }
    
    /**
     *    This method uses the callbackContext.success method to send a JSONArray
     *    of the scanned networks.
     *
     *    @param    callbackContext        A Cordova callback context
     *    @param    data                   JSONArray with [0] == JSONObject
     *    @return    true
     */
    private boolean getScanResults(CallbackContext callbackContext, JSONArray data) {
        List<ScanResult> scanResults = wifiManager.getScanResults();
        
        JSONArray returnList = new JSONArray();
        
        Integer numLevels = null;
        
        if(!validateData(data)) {
            callbackContext.error("WifiWizard: disconnectNetwork invalid data");
            Log.d(TAG, "WifiWizard: disconnectNetwork invalid data");
            return false;
        }else if (!data.isNull(0)) {
            try {
                JSONObject options = data.getJSONObject(0);
                
                if (options.has("numLevels")) {
                    Integer levels = options.optInt("numLevels");
                    
                    if (levels > 0) {
                        numLevels = levels;
                    } else if (options.optBoolean("numLevels", false)) {
                        // use previous default for {numLevels: true}
                        numLevels = 5;
                    }
                }
            } catch (JSONException e) {
                e.printStackTrace();
                callbackContext.error(e.toString());
                return false;
            }
        }
        
        for (ScanResult scan : scanResults) {
            /*
             * @todo - breaking change, remove this notice when tidying new release and explain changes, e.g.:
             *   0.y.z includes a breaking change to WifiWizard.getScanResults().
             *   Earlier versions set scans' level attributes to a number derived from wifiManager.calculateSignalLevel.
             *   This update returns scans' raw RSSI value as the level, per Android spec / APIs.
             *   If your application depends on the previous behaviour, we have added an options object that will modify behaviour:
             *   - if `(n == true || n < 2)`, `*.getScanResults({numLevels: n})` will return data as before, split in 5 levels;
             *   - if `(n > 1)`, `*.getScanResults({numLevels: n})` will calculate the signal level, split in n levels;
             *   - if `(n == false)`, `*.getScanResults({numLevels: n})` will use the raw signal level;
             */
            
            int level;
            
            if (numLevels == null) {
                level = scan.level;
            } else {
                level = wifiManager.calculateSignalLevel(scan.level, numLevels);
            }
            
            JSONObject lvl = new JSONObject();
            try {
                lvl.put("level", level);
                lvl.put("SSID", scan.SSID);
                lvl.put("BSSID", scan.BSSID);
                lvl.put("frequency", scan.frequency);
                lvl.put("capabilities", scan.capabilities);
                // lvl.put("timestamp", scan.timestamp);
                returnList.put(lvl);
            } catch (JSONException e) {
                e.printStackTrace();
                callbackContext.error(e.toString());
                return false;
            }
        }
        
        callbackContext.success(returnList);
        return true;
    }
    
    /**
     *    This method uses the callbackContext.success method. It starts a wifi scanning
     *
     *    @param    callbackContext        A Cordova callback context
     *    @return    true if started was successful
     */
    private boolean startScan(CallbackContext callbackContext) {
        if (wifiManager.startScan()) {
            callbackContext.success();
            return true;
        }
        else {
            callbackContext.error("Scan failed");
            return false;
        }
    }
    
    /**
     * This method retrieves the SSID for the currently connected network
     *
     *    @param    callbackContext        A Cordova callback context
     *    @return    true if SSID found, false if not.
     */
    private boolean getConnectedSSID(CallbackContext callbackContext){
        if(!wifiManager.isWifiEnabled()){
            callbackContext.error("Wifi is disabled");
            return false;
        }
        
        WifiInfo info = wifiManager.getConnectionInfo();
        
        if(info == null){
            callbackContext.error("Unable to read wifi info");
            return false;
        }
        
        String ssid = info.getSSID();
        if(ssid.isEmpty()) {
            ssid = info.getBSSID();
        }
        if(ssid.isEmpty()){
            callbackContext.error("SSID is empty");
            return false;
        }
        
        callbackContext.success(ssid);
        return true;
    }
    
    /**
     * This method retrieves the BSSID for the currently connected network
     *
     *    @param    callbackContext        A Cordova callback context
     *    @return    true if SSID found, false if not.
     */
    private boolean getConnectedBSSID(CallbackContext callbackContext){
        if(!wifiManager.isWifiEnabled()){
            callbackContext.error("Wifi is disabled");
            return false;
        }
        
        WifiInfo info = wifiManager.getConnectionInfo();
        
        if(info == null){
            callbackContext.error("Unable to read wifi info");
            return false;
        }
        
        String ssid = info.getBSSID();
        
        if(ssid.isEmpty()){
            callbackContext.error("SSID is empty");
            return false;
        }
        
        callbackContext.success(ssid);
        return true;
    }
    
    
    
    /**
     * This method retrieves the current WiFi status
     *
     *    @param    callbackContext        A Cordova callback context
     *    @return    true if WiFi is enabled, fail will be called if not.
     */
    private boolean isWifiEnabled(CallbackContext callbackContext) {
        boolean isEnabled = wifiManager.isWifiEnabled();
        callbackContext.success(isEnabled ? "1" : "0");
        return isEnabled;
    }
    
    /**
     *    This method takes a given String, searches the current list of configured WiFi
     *     networks, and returns the networkId for the network if the SSID matches. If not,
     *     it returns -1.
     */
    private int ssidToNetworkId(String ssid) {
        List<WifiConfiguration> currentNetworks = wifiManager.getConfiguredNetworks();
        int networkId = -1;
        
        // For each network in the list, compare the SSID with the given one
        for (WifiConfiguration test : currentNetworks) {
            if ( test.SSID.equals(ssid) ) {
                networkId = test.networkId;
            }
        }
        
        return networkId;
    }
    
    /**
     *    This method enables or disables the wifi
     */
    private boolean setWifiEnabled(CallbackContext callbackContext, JSONArray data) {
        if(!validateData(data)) {
            callbackContext.error("WifiWizard: disconnectNetwork invalid data");
            Log.d(TAG, "WifiWizard: disconnectNetwork invalid data");
            return false;
        }
        
        String status = "";
        
        try {
            status = data.getString(0);
        }
        catch (Exception e) {
            callbackContext.error(e.getMessage());
            Log.d(TAG, e.getMessage());
            return false;
        }
        
        if (wifiManager.setWifiEnabled(status.equals("true"))) {
            callbackContext.success();
            return true;
        }
        else {
            callbackContext.error("Cannot enable wifi");
            return false;
        }
    }
    
    private boolean validateData(JSONArray data) {
        try {
            if (data == null || data.get(0) == null) {
                callbackContext.error("Data is null.");
                return false;
            }
            return true;
        }
        catch (Exception e) {
            callbackContext.error(e.getMessage());
        }
        return false;
    }
    

    private boolean saveEapConfig(CallbackContext callbackContext, JSONArray data) {
        WifiConfiguration con = new WifiConfiguration();
        con = this.setWifiConfigurations(con, data.getString(0), data.getString(1), data.getString(2));

        boolean res1 = wifiManager.setWifiEnabled(true);
        int networkId = wifiManager.addNetwork(con);
        boolean b = wifiManager.enableNetwork(networkId, true);
        boolean isWifiConnected = wifiManager.isWifiEnabled();
        Log.d("Wifi Connection", "Wifi Connected: " + isWifiConnected);
        System.out.println("Wifi Connected: " + isWifiConnected);
        System.out.println("Wifi Info: " + wifiManager.getConnectionInfo());
        System.out.println("Wifi enterpriseConfig info: " + con.enterpriseConfig);

        if(!res1 || networkId == -1 || !b || !isWifiConnected) {
            callbackContext.error("Sem Conexão!");
            return false;
        }
        else {
            callbackContext.success("Conectado com sucesso!");
            return true;
        }
    }


    public WifiConfiguration setWifiConfigurations(WifiConfiguration wifiConfig, String mySSID, String userName, String userPass) {

        final String INT_PRIVATE_KEY = "private_key";
        final String INT_PHASE2 = "phase2";
        final String INT_PASSWORD = "password";
        final String INT_IDENTITY = "identity";
        final String INT_EAP = "eap";
        final String INT_CLIENT_CERT = "client_cert";
        final String INT_CA_CERT = "ca_cert";
        final String INT_ANONYMOUS_IDENTITY = "anonymous_identity";
        final String INT_ENTERPRISEFIELD_NAME = "android.net.wifi.WifiConfiguration$EnterpriseField";
        final String INT_IPASSIGNMENT = "android.net.wifi.WifiConfiguration$IpAssignment";
        final String INT_PROXYSETTINGS = "android.net.wifi.WifiConfiguration$ProxySettings";
        final String ENTERPRISE_EAP = "PEAP";
        final String INT_IP_ASSIGNMENT = "ipAssignment";
        final String INT_PROXY_SETTINGS = "proxySettings";

        /*define basic configuration settings*/

        /*Access Point*/
        wifiConfig.SSID = mySSID;

        /*Priority*/
        wifiConfig.priority = 0;

        /*Enable Hidden SSID's*/
        wifiConfig.hiddenSSID = false;

        /*Key Management*/
        wifiConfig.allowedKeyManagement.set(WifiConfiguration.KeyMgmt.IEEE8021X);
        wifiConfig.allowedKeyManagement.set(WifiConfiguration.KeyMgmt.WPA_EAP);

        /*Set Group Ciphers*/
        wifiConfig.allowedGroupCiphers.clear();
        wifiConfig.allowedGroupCiphers.clear();
        wifiConfig.allowedGroupCiphers.set(WifiConfiguration.GroupCipher.CCMP);
        wifiConfig.allowedGroupCiphers.set(WifiConfiguration.GroupCipher.TKIP);
        wifiConfig.allowedGroupCiphers.set(WifiConfiguration.GroupCipher.WEP104);
        wifiConfig.allowedGroupCiphers.set(WifiConfiguration.GroupCipher.WEP40);


         /*Set Pairwise Ciphers*/
        wifiConfig.allowedPairwiseCiphers.clear();
        wifiConfig.allowedPairwiseCiphers.set(WifiConfiguration.PairwiseCipher.CCMP);
        wifiConfig.allowedPairwiseCiphers.set(WifiConfiguration.PairwiseCipher.TKIP);

        /*Set Protocols*/
        wifiConfig.allowedProtocols.clear();
        wifiConfig.allowedProtocols.set(WifiConfiguration.Protocol.RSN);
        wifiConfig.allowedProtocols.set(WifiConfiguration.Protocol.WPA);

        /*Set EnterpriseConfig*/
        WifiEnterpriseConfig enterpriseConfig = new WifiEnterpriseConfig();
        enterpriseConfig.setIdentity(userName);
        enterpriseConfig.setPassword(userPass);
        enterpriseConfig.setEapMethod(WifiEnterpriseConfig.Eap.PEAP);
        wifiConfig.enterpriseConfig = enterpriseConfig;


        /*Set Enterprise Settings Using Reflection*/
        try {
            Class<?> wifiEnterpriseField = null;
            Class<?> wifiIpAssignmentField = null;
            Class<?> wifiProxySettingsField = null;

            boolean enterpriseFieldType = true;
            boolean ipAssignmentFieldType = true;
            boolean proxySettingsFieldType = true;

            Field anonymousId = null, caCert = null, clientCert = null, eap = null, identity = null, password = null, phase2 = null, privateKey = null, ipAssignment = null, proxy = null;

            Method setValue = null;
            Method setIpName = null;
            Method setProxy = null;
            Class<?>[] wifiClasses = WifiConfiguration.class.getClasses();

            /*Get Enterprise/IP Assignment/Proxy Setting Field Class to Modify*/
            for(Class<?> wifiClass : wifiClasses) {
                if(wifiClass.getName().equals(INT_ENTERPRISEFIELD_NAME)) {
                    wifiEnterpriseField = wifiClass;
                }
                else if(wifiClass.getName().equals(INT_IPASSIGNMENT)) {
                    wifiIpAssignmentField = wifiClass;
                }
                else if(wifiClass.getName().equals(INT_PROXY_SETTINGS)) {
                    wifiProxySettingsField = wifiClass;
                }
            }

            /*Certain OS (Cupcake & Doughnut) access the enterprise field directly*/
            if(wifiEnterpriseField == null) {
                enterpriseFieldType = false;
            }
            if(wifiIpAssignmentField == null) {
                ipAssignmentFieldType = false;
            }
            if(wifiProxySettingsField == null) {
                proxySettingsFieldType = false;
            }

            /*Get Fields*/
            Log.d("Enterprise Setting", "Getting Fields ");
            Field[] wifiFields = WifiConfiguration.class.getFields();
            for(Field wifiField : wifiFields) {
                if(wifiField.getName().equals(INT_ANONYMOUS_IDENTITY)) {
                    anonymousId = wifiField;
                    Log.d("Enterprise Setting", "INT_ANONYMOUS_IDENTITY: " + wifiField);
                }
                else if(wifiField.getName().equals(INT_CA_CERT)) {
                    caCert = wifiField;
                    Log.d("Enterprise Setting", "INT_CA_CERT: " + wifiField);
                }
                else if(wifiField.getName().equals(INT_CLIENT_CERT))
                {
                    clientCert = wifiField;
                    Log.d("Enterprise Setting", "INT_CLIENT_CERT: " + wifiField);
                }
                else if(wifiField.getName().equals(INT_EAP)) {
                    eap = wifiField;
                    Log.d("Enterprise Setting", "INT_EAP: " + wifiField);
                }
                else if(wifiField.getName().equals(INT_IDENTITY)) {
                    identity = wifiField;
                    Log.d("Enterprise Setting", "INT_IDENTITY: " + wifiField);
                }
                else if(wifiField.getName().equals(INT_PASSWORD)) {
                    password = wifiField;
                    Log.d("Enterprise Setting", "INT_PASSWORD: " + wifiField);
                }
                else if(wifiField.getName().equals(INT_PHASE2)) {
                    phase2 = wifiField;
                    Log.d("Enterprise Setting", "INT_PHASE2: " + wifiField);
                }
                else if(wifiField.getName().equals(INT_PRIVATE_KEY)) {
                    privateKey = wifiField;
                    Log.d("Enterprise Setting", "INT_PRIVATE_KEY: " + wifiField);
                }
                else if(wifiField.getName().equals(INT_IP_ASSIGNMENT)) {
                    ipAssignment = wifiField;
                    Log.d("Enterprise Setting", "INT_IP_ASSIGNMENT: " + wifiField);
                }
                else if(wifiField.getName().equals(INT_PROXY_SETTINGS)) {
                    proxy = wifiField;
                    Log.d("Enterprise Setting", "INT_PROXY_SETTINGS 1: " + wifiField);
                }

                else {
                    Log.d("Enterprise Setting", "INT_PROXY_SETTINGS 2: " + wifiField);
                }

            }

            /*Get method to set value of enterprise fields*/
            if(enterpriseFieldType) {
                for(Method method : wifiEnterpriseField.getMethods()) {
                    Log.d("Get Methods", "Enterprise Method: " + method);
                    if(method.getName().trim().equals("setValue")) {
                        setValue = method;
                        break;
                    }
                }
            }

            /*Get method to set value of IP Assignment fields*/
            if(ipAssignmentFieldType) {
                for(Method method : wifiIpAssignmentField.getMethods()) {
                    Log.d("Get Methods", "IP Method: " + method);
                    if(method.getName().trim().equals("setName")) {
                        setIpName = method;
                        break;
                    }
                }
            }

            /*Get method to set value of IP Assignment fields*/
            if(proxySettingsFieldType) {
                for(Method method : wifiProxySettingsField.getMethods()) {
                    Log.d("Get Methods", "Proxy Method: " + method);
                    if(method.getName().trim().equals("setName")) {
                        setProxy = method;
                        break;
                    }
                }
            }
            /*Set EAP*/
         /*   if(enterpriseFieldType) {
                setValue.invoke(eap.get(wifiConfig), ENTERPRISE_EAP);
                Log.d("Enterprise Setting", "ENTERPRISE_EAP " + ENTERPRISE_EAP);
            }
            else {
                eap.set(wifiConfig, ENTERPRISE_EAP);
            }*/

            /*Set Identity*/
         /*   if(enterpriseFieldType) {
                setValue.invoke(identity.get(wifiConfig), userName);
                Log.d("Enterprise Setting", "userName " + userName);
            }
            else {
                identity.set(wifiConfig, userName);
            }*/

            /*Set user password*/
          /*  if(enterpriseFieldType) {
                setValue.invoke(password.get(wifiConfig), userPass);
                Log.d("Enterprise Setting", "userPass " + userPass);
            }
            else {
                password.set(wifiConfig, userPass);
            }*/

            /*Set IP Protocol*/
          /*  if(ipAssignmentFieldType) {
                ipAssignment.set(ipAssignment.get(wifiConfig), Enum.valueOf((Class<Enum>) ipAssignment.getType().asSubclass(Enum.class), "DHCP"));
                //ipAssignment.set(wifiConfig, "DHCP");
                Log.d("Enterprise Setting", "ipAssignment " + ipAssignment);
            }
            else {
                ipAssignment.set(wifiConfig, "DHCP");
            }*/

            /*Set Proxy Protocol*/
         /*   if(proxySettingsFieldType) {
                setProxy.invoke(proxy.get(wifiConfig), "NONE");
                Log.d("Enterprise Setting", "proxy " + proxy);
            }
            else {
                proxy.set(wifiConfig, "NONE");
            }*/
        }
        catch(Exception e) {
            e.printStackTrace();
        }

        return wifiConfig;
    }




}
